function[mloga,vloga]=Linear(x,y,ii);
%%%Function to compute posterior distributions of the parameters in the 'linear'
%model and the posterior predictive 'linear' model,which are used in the paper 
%"A semiparametric Bayesian approach to estimating maximum reproductive rates 
%at low population sizes" (M.Sugeno and S.B. Munch).

%%%Inputs:
%'x' is a column vector for stock size(s) rescaled for analysis,x=s/max(s).
%'y' is a column vector for a transformed variate describing a
%stock-recruitment relationship,y=log(r/s),where 'r' is a column vector for
%recruitment.
%'ii' is a scalar determining number of iterations in the MCMC routine.In 
%this paper ii=20000 is used.When there is a trouble obtaining a result,the 
%number of iteration can be made longer.

%%%Outputs:
%This function returns an estimate of 'log(alpha)',where 'alpha' is reproductive 
%rate estimated at S=0.
%'mloga' is a mean of a posterior distribution of 'log(alpha)'.
%'vloga' is a variance of a posterior distribution of 'log(alpha)'.

%%%Results stored in a separate file 'posterior and posterior predictive':
%This function does not output results that are not relevant to the main results
%presented in this paper.These results are stored in a separate MAT-file 
%'posterior and posterior predictive'.
%'loga' is a vector collecting samples generated from a posterior distribution 
%of 'log(alpha)'.
%'be' is a vector collecting samples generated from a posterior
%distribution of 'beta'.
%'logphi' is a vector collecting samples generated from a posterior
%distribution of 'log(phi)'.
%'logtau' is a vector collecting samples generated from a posterior
%distribution of 'log(tau squared)'.
%'logv' is a vector collecting samples generated from a posterior
%distribution of 'log(sigma squared)'.
%'xnew' is a vector used for generating samples from a posterior predictive
%'linear' model.The default is xnew=linspace(0,1,30)'.If you want a finer 
%grid for the posterior predictive,30 can be made larger.If you want to 
%inspect the posterior predictive outside the range of data,1 can be made 
%larger.
%'fmean' is a vector for a mean of the posterior predictive 'linear' model.
%'fup' is a vector for the upper 97.5% credibility bound calculated for the 
%posterior predictive 'linear' model.
%'fdown' is a vector for the lower 2.5% credibility bound calculated for the 
%posterior predictive 'linear' model.
%'x' is a vector for data,x=s/max(s).
%'y' is a vector for data,y=log(r/s).

%%%Plots:
%This function plots followings:
%'x' and 'y' are drawn with black circle.
%'fmean' is drawn with black dashed line.
%'fup' and 'fdown' are drawn with red dashed line.
%Posterior distributions of parameters in the 'Linear' model.

%%%Notes:
%In this function,'log(alpha)' is written as 'loga','log(phi)' is written as 
%'logphi','log(tau2)' is written as 'logtau',and 'log(sigma squared)' is 
%written as 'logv'.
%To execute the model,prepare a data file in MAT format named as 'test_data'
%in the same directory as this function,copy and past the line that follows 
%this sentence,and hit a command 'save and run'. 
%clear all;close all;load 'test_data';ii=20000;x=s/max(s);y=log(r./s);[mloga,vloga]=Linear(x,y,ii);



%%%Description of the 'linear' model 
%Here, c1 and c2 are the numbers used as criteria in assessing convergence of 
%posterior distributions(20-30% is a reasonable range,see m-file titled 'convergence').
c1=0.2;
c2=0.3;

%Defining the initial jump size used in the MCMC routine(this jump size is 
%adjusted automatically in the MCMC routine so that the acceptance rate can 
%approaches to 20-30%).
j=0.5;

%Counting the number of accepted iterations ('kept' is used to see if acceptance 
%rate is 20-30%).
kept=0;

%Constants used in defining vectors and covariances used in the covariance
%function.
al=2;

%Defining vectors and maxtices that are used for constructing the mean and
%the covariance functions.
lx=length(x);
ix=eye(lx);
ly=length(y);
iy=eye(ly);
logavec=ones(lx,1);
xd=abs(x*ones(1,lx)-ones(lx,1)*x').^al;
xvec=x.^al;

%Defining hyperparameters specifying some of the prior distributions of parameters 
%in the 'linear' model:gamma distributions for phi and tau,which are used in log scale.
ry=max(y)-min(y);
optphi=4;
alpphi=-1;bepphi=1/optphi;
alptau=-1;beptau=1/(1/2*ry);

%Defining vectors and maxrices that are used for constructing the posterior
%predictive 'linear' model.
xnew=linspace(0,1,30)';
lxnew=length(xnew);
xnewd=abs(xnew*ones(1,lxnew)-ones(lxnew,1)*xnew').^al;
xnewvec=xnew.^al;
xnd=abs(xnew*ones(1,lx)-ones(lxnew,1)*x').^al;

%Identifying maximum a posteri (MAP) estimates of parameters in the 'linear' model. 
%They are used for defining initial guesses in the MCMC routine. See a
%function 'Linear_MAP' for the source code.
pars0=[mean(y),log(optphi),log(1/2*ry),log(1/2*ry),0];
pars1=[x,y];
pars2=[alpphi,bepphi];
pars3=[alptau,bepphi];
mle=fminsearch(@Linear_MAP,pars0,[],pars1,pars2,pars3);
ini=mle;

%Setting initial values for the MCMC routine (they are dispersed around the 
%MAP estimates).We run two different MCMC for diagnosing convergence of the 
%posterior distributions.
start(:,:,1)=1.1*ini;start(:,:,2)=1/1.1*ini;

%Constructing posterior distributions of 'logphi','logtau', and
%'logv' via MCMC(Metropolis step).Eigenvalue decomposition is used for
%sampling from multivariate normal.The samples are used as proposal samples.
D=eye(length(3));
[T,l]=eig(D);
rotD(:,:,1)=j*l^(1/2)*T';
rotD(:,:,2)=j*l^(1/2)*T';

%One of two convergence diagnosis:acceptance rate approach.
%'pass=[0,0]' indicates that the samples do not converge. 'pass' becomes 1 
%if acceptance rate is 20-30%.
pass=[0,0];
%The other convergence diagnosis:see 'Bayesian data analysis' by Gelman et al 
%2003 and the source code 'convergence'. 
%'grandpass' becomes 1 if comparison of two different runs MCMC indicates 
%that the posterior distributions are converged and the acceptance rates are
%20-30%.
grandpass=0;
while grandpass==0;
    %Starting two different runs.
    for k=1:2;
        m=start(:,:,k);
        %Running the MCMC routine till the acceptance rates become 20-30%.
        while pass(k)==0;
            for i=1:ii;
                %Initial values for one of the two MCMC runs.
                logphi=m(i,2);logtau=m(i,3);logv=m(i,4);

                %Constructing a covariance function for the 'linear' model.
                co=exp(logtau)*exp(-exp(logphi)*xd);
                co_cond=co-exp(logtau)*exp(-exp(logphi)*xvec)*exp(-exp(logphi)*xvec');    
                
                %Constructing posterior distributions for 'loga' and 'beta'
                %within Gibbs step ('MU' is the mean vector and 'C' is the 
                %covariance matrix).
                X=[logavec,x];
                C=inv(X'*inv(co_cond+exp(logv)*iy)*X);
                MU=transpose(y'*inv(co_cond+exp(logv)*iy)*X*C) ;
                %Eigenvalue decomposition for sampling 'loga' and 'beta' 
                %from multivariate normal.
                [F,L]=eig(C);
                rotF=L^(1/2)*F';
                th=MU+rotF*randn(2,1);
                loga=th(1);
                be=th(2);
                m(i+1,1)=loga;
                m(i+1,5)=be;
                
                %Constructing posterior distribution for for 'logphi','logtau',
                %and 'logv' via Metropolis step.
                %Likelihood(old).
                Lold=-.5*(y-loga-be*x)'*inv(co_cond+exp(logv)*iy)*(y-loga-be*x)-.5*log(det(co_cond+exp(logv)*iy));
                %Prior(old)(flat for 'loga' and 'beta').
                Plogphi=log((exp(logphi)).^(-alpphi).*exp(-bepphi.*(exp(logphi))));
                Plogtau=log((exp(logtau)).^(-alptau).*exp(-beptau.*(exp(logtau))));
                Plogv=0;%flat prior
                Lold=Lold+Plogphi+Plogtau+Plogv;
                %Generating proposal samples of the posterior distributions.
                m(i+1,2:4)=randn(1,3)*rotD(:,:,k)+m(i,2:4);
                logphi=m(i+1,2);
                logtau=m(i+1,3);
                logv=m(i+1,4);
                %Updating the likelihood.
                co=exp(logtau)*exp(-exp(logphi)*xd);
                co_cond=co-exp(logtau)*exp(-exp(logphi)*xvec)*exp(-exp(logphi)*xvec');             
                Lnew=-.5*(y-loga-be*x)'*inv(co_cond+exp(logv)*iy)*(y-loga-be*x)-.5*log(det(co_cond+exp(logv)*iy));
                %Updating the prior distributions(flat for 'loga' and 'beta').
                Plogphi=log((exp(logphi)).^(-alpphi).*exp(-bepphi.*(exp(logphi))));
                Plogtau=log((exp(logtau)).^(-alptau).*exp(-beptau.*(exp(logtau))));
                Plogv=0;%flat prior
                Lnew=Lnew+Plogphi+Plogtau+Plogv;
                %Determining acceptance/rejection of the proposal samples.
                ratio=Lnew-Lold;
                keep=ratio>=log(rand(1));
                m(i+1,2:4)=m(i+1,2:4)*keep+m(i,2:4)*(1-keep);
                kept=kept+keep;
            end
            %Calculating the acceptance rate (has to be 20-30%).
            rate(k)=kept/ii;
            pass1(k)=rate(k)>c1;
            pass2(k)=rate(k)<c2;
            pass(k)=pass1(k)*pass2(k);
            %If the acceptance rate is too low(high),jump size is
            %reduced(increased).
            switch pass(k)
                case 0
                    rotD(:,:,k)=0.8^(1-pass1(k))*1.2^(1-pass2(k))*rotD(:,:,k);               
                    ini=m(end,:);
                    m=ini;
                    kept=0;
            end
        end
        %Collecting samples from posterior distributions.
        test(:,:,k)=m;
        loga=m(:,1);
        be=m(:,5);
        logphi=m(:,2);
        logtau=m(:,3);
        logv=m(:,4);        
        %Resetting convegence diagnosis in terms of acceptance ratio to
        %the default.
        pass=[0,0];
        clear m;
    end

    %Convergence diagnosis in terms of 'Bayesian data analytsis' by Gelman 
    %et al 2003.The diagnosis test is conducted for all the 5 parameters in 
    %the 'linear' model (see a separate file titled 'convergence' for the 
    %source code).
    for l=1:5;      
        for o=1:2;         
            conv(:,o)=test(:,l,o);
        end
        R(l)=convergence(conv,1);
        clear conv;
    end
    %R has to be <1.1 for each parameter.All the parameters have to pass the criterion.
    Rup=1.1;
    R=sum(R>Rup);
    grandpass=(R<1);
    start(1,:,k)=test(end,:,k); 
end
clear test

%Constructing posterior predictive 'linear' model.
%Discarding the first half of samples for avoiding including burn-in
%samples. 
loga=loga(ii/2+1:end,1);
be=be(ii/2+1:end,1);
logphi=logphi(ii/2+1:end,1);
logtau=logtau(ii/2+1:end,1);
logv=logv(ii/2+1:end,1);

%Calculation of the outputs.
mloga=mean(loga);
vloga=var(loga);

%Plotting posterior distributions of the parameters in the 'linear' model.
figure;
subplot(5,2,2);plot(loga);ylabel('log(alpha)');
subplot(5,2,4);plot(be);ylabel('beta');
subplot(5,2,6);plot(logphi);ylabel('log(phi)');
subplot(5,2,8);plot(logtau);ylabel('log(tau^2)');
subplot(5,2,10);plot(logv);ylabel('log(sigma^2)');

%Monte carlo integration for generating the posterior predictive 'linear' model.
ii=length(loga);
for i=1:ii;
    %Constructing matrices for a covariance function.
    co_new=exp(logtau(i))*exp(-exp(logphi(i))*xnewd); 
    co_new=co_new-exp(logtau(i))*exp(-exp(logphi(i))*xnewvec)*exp(-exp(logphi(i))*xnewvec');
    co_old=exp(logtau(i))*exp(-exp(logphi(i))*xd);       
    co_old=co_old-exp(logtau(i))*exp(-exp(logphi(i))*xvec)*exp(-exp(logphi(i))*xvec');
    co_nd=exp(logtau(i))*exp(-exp(logphi(i))*xnd);
    co_nd=co_nd-exp(logtau(i))*exp(-exp(logphi(i))*xnewvec)*exp(-exp(logphi(i))*xvec');

    %Constructing mean and covariance functions of the posterior
    %predictive 'linear' model.
    mupost=loga(i)+be(i)*xnew+co_nd*inv(co_old+exp(logv(i))*iy)*(y-(loga(i)+be(i)*x));
    copost=co_new-co_nd*inv(co_old+exp(logv(i))*iy)*co_nd';

    %Eigenvalue decomposition for sampling from a multivariate normal.
    %We sampled the postetior predictive 'linear' model in this way.
    [T,l]=eig(copost);rotD=l^(1/2)*T';

    %Sampling from the posterior predictive 'linear' model
    fpost(:,i)=mupost+rotD*randn(lxnew,1);
end
%Plotting samples of the posterior predictive 'linear' model to check
%performance of the 'linear' model using the mean,the 97.5% upper bound,and 
%the 2.5% lower bound.
subplot(1,2,1);
fmean=mean(real(fpost),2);
fsort=sort(real(fpost),2);
fup=fsort(:,ceil(0.975*ii));
fdown=fsort(:,floor(0.025*ii));
plot(x,y,'bO');hold on;
plot(xnew,fmean,'k:');hold on;
plot(xnew,fup,'r:');hold on;
plot(xnew,fdown,'r:')
xlabel('S/max(S)');
ylabel('log(R/S)');

%Saving the posterior distributions of paramters,the posterior predictive
%'linear' model,and data (x=s/max(s),y=log(r./s)), in a filed titled 
%'posterior and posterior predictive'.
save('posterior and posterior predictive','loga','logphi','logtau','logv','be','xnew','fmean','fup','fdown','x','y');